import {
  ClassType,
  PropertyKeyType,
  Metadata,
} from '@overflow/commons/core/type';

import {AccessibleObject} from './accessible_object';
import {systemClassRegistry} from './class_registry';
import {Constructor} from './constructor';
import {Field} from './field';
import {Method} from './method';



export class Class extends AccessibleObject {
  private _clazzType: ClassType;
  private _constructor: Constructor;
  private _fields: Map<PropertyKeyType, Field>;
  private _methods: Map<PropertyKeyType, Method>;

  /**
   * forClass
   */
  public static forClass(clazzType: ClassType): Class {
    return systemClassRegistry.get(clazzType);
  }

  /**
   * _defineClass
   */
  public static _defineClass(clazzType: ClassType): Class {
    let clazz: Class = Class.forClass(clazzType);
    if (undefined === clazz) {
      clazz = new Class(clazzType);
      systemClassRegistry.set(clazzType, clazz);
    }

    if (null === clazz._constructor) {
      let parameterTypes = Metadata.getOwnParamTypes(clazzType);
      if (undefined !== parameterTypes) {
        clazz._constructor = new Constructor(clazz, parameterTypes);
      }
    }

    return clazz;
  }

  private constructor(clazzType: ClassType) {
    super();
    this._clazzType = clazzType;
    this._fields = new Map();
    this._methods = new Map();
    this._constructor = null;
  }

  /**
   * _defineField
   */
  public _defineConstructor(parameterTypes: any[]): Constructor {
    let cons: Constructor = this._constructor;
    if (undefined === cons) {
      cons = new Constructor(this, parameterTypes);
      this._constructor = cons;
    }
    return cons;
  }

  /**
   * _defineField
   */
  public _defineField(propertyKey: PropertyKeyType, propertyType: any): Field {
    let field: Field = this._fields.get(propertyKey);
    if (undefined === field) {
      field = new Field(this, propertyKey, propertyType);
      this._fields.set(propertyKey, field);
    }
    return field;
  }

  /**
   * _defineMethod
   */
  public _defineMethod(propertyKey: PropertyKeyType, parameterTypes: any[], returnType: any): Method {
    let method: Method = this._methods.get(propertyKey);
    if (undefined === method) {
      method = new Method(this, propertyKey, parameterTypes, returnType);
      this._methods.set(propertyKey, method);
    }
    return method;
  }

  public getType(): ClassType {
    return this._clazzType;
  }

  public getConstructor(): Constructor {
    return this._constructor;
  }

  public getOwnField(propertyKey: PropertyKeyType): Field {
    return this._fields.get(propertyKey);
  }

  public getOwnFields(): Map<PropertyKeyType, Field> {
    return this._fields;
  }

  public getField(propertyKey: PropertyKeyType): Field {
    return null;
  }

  public getFields(): Field[] {
    return null;
  }

  public getOwnMethod(propertyKey: PropertyKeyType): Method {
    return this._methods.get(propertyKey);
  }

  public getOwnMethods(): Map<PropertyKeyType, Method> {
    return this._methods;
  }

  public getMethod(propertyKey: PropertyKeyType): Method {
    return null;
  }

  public getMethods(): Method[] {
    return null;
  }

  public getName(): string {
    return null;
  }
}
